// Teppich.java
/*
 * This is a Sierpinski fractal.
 * With probability 1/8 do the following:
 * divides the current (x, y) value by 3
 * and may add either 1/3 or 2/3 to x, or to y, or to both.
 * In order to create the Sierpinski carpet holes,
 * give zero probability to the possibility
 * xnew = x/3 + 1.0/3;
 * xnew = y/3 + 1.0/3;
 */

import ch.aplu.util.*;
import java.awt.Color;

class Teppich
{
  public static void main(String[] args)
  {
    final int MAXITERATIONS = 200000;
    GPanel p = new GPanel(0, 420, 0, 420);
    p.color(Color.blue);

    double x = 1, y = 1, xnew = 1, ynew = 1;
    double hazard, xplot, yplot;

    for (int iter = 0; iter <= MAXITERATIONS; iter++)
    {
      hazard = 8 * Math.random();
      if (hazard < 1)
      {
        xnew = x / 3;
        ynew = y / 3;
      }
      else
      {
        if (hazard < 2)
        {
          xnew = x / 3 + 2.0 / 3;
          ynew = y / 3 + 2.0 / 3;
        }
        else
        {
          if (hazard < 3)
          {
            xnew = x / 3 + 2.0 / 3;
            ynew = y / 3 + 1.0 / 3;
          }
          else
          {
            if (hazard < 4)
            {
              xnew = x / 3 + 1.0 / 3;
              ynew = y / 3 + 2.0 / 3;
            }
            else
            {
              if (hazard < 5)
              {
                xnew = x / 3 + 1.0 / 3;
                ynew = y / 3;
              }
              else
              {
                if (hazard < 6)
                {
                  xnew = x / 3;
                  ynew = y / 3 + 1.0 / 3;
                }
                else
                {
                  if (hazard < 7)
                  {
                    xnew = x / 3 + 2.0 / 3;
                    ynew = y / 3;
                  }
                  else
                  {
                    xnew = x / 3;
                    ynew = y / 3 + 2.0 / 3;
                  }
                }
              }
            }
          }
        }
      }
      xplot = 10 + 400 * xnew;
      yplot = 10 + 400 * ynew;
      p.point(xplot, yplot);
      x = xnew;
      y = ynew;
    }
    p.color(Color.black);
    p.lineWidth(2);
    p.pos(210, 210);
    p.rectangle(400, 400);
  }
}



