// SimEx4.java
// Pendelfeder: Masse an einer Feder aufgehaengt, die sich wie ein Pedelstab
// bewegen kann (bleibt gerade)

import ch.aplu.util.*;
import static java.lang.Math.*;

class SimEx4 extends GPanel
{
  SimEx4()
  {
   final double g = 9.81;             // Erdbeschleunigung  in m/s^2
   final double d = 0.001;            // Zeitschritt in s
   final double m = 1;                // Kugelmasse in kg
   final double k = 3;                // Federkonstante in N/m
   final double l = 1.4;              // Ruhelaenge der Feder in m
   final double x0 = 3;               // Anfangskoordinate x in m
   final double y0 = 1;               // Anfangskoordinate y in m
   final double vx0 = 0;              // Anfangsgeschw. vx in m/s
   final double vy0 = 0;              // Anfangsgeschw. vy in m/s


   double t = 0;
   double x = x0, y = y0;
   double vx = vx0, vy = vy0;
   double ax, ay, fx, fy, r;

   window(-5, 5, 10, 0);
   line(0, 0, 0, 10);       // y-Achse
   text(-4.8, 0.6, String.
     format("Aequivalente Federperiode:  %5.2f s", 2*PI*sqrt(m/k)));
   text(-4.8, 1, String.
     format("Aequivalente Pendelperiode: %5.2f s", 2*PI*sqrt((l+m*g/k)/g)));

   pos(x0, y0);
   int it = 0;  // Interationszahl
   LoResAlarmTimer timer = new LoResAlarmTimer(100000, true);
   do
   {
      r = sqrt( x*x + y*y );                // Radius
      fx = -k * (r - l) * x / r;            // Kraft
      fy = -k * (r - l) * y / r + m * g;

      ax = fx/m;                            // Beschleunigung
      ay = fy/m;

      vx = vx + ax*d;                       // Geschwindigkeit
      vy = vy + ay*d;

      x = x + vx*d;                         // Ortskoordinaten
      y = y + vy*d;

      t = t + d;                            // Zeit

      if (it++ % 100 == 0)                  // Zeichne nur alle 100 Iterationen
      {
        while (timer.isRunning()) {}  // Auskommentieren zeigt
        timer.start();                // schlechtes Zeitverhalten
        title(String.format("Pendelfeder. Zeit t = %5.1f s", t));
        draw(x, y);
      }
   } while (t < 10);
  }


  public static void main(String[] args)
  {
    new SimEx4();
  }
}
