// SelectionPalette

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import ch.aplu.util.Monitor;

class SelectionPalette extends JDialog
{
  private class RadioBtnActionAdapter
          implements ActionListener
  {
    private int btnNb;

    private RadioBtnActionAdapter(int btnNb)
    {
      this.btnNb = btnNb;
    }

    public void actionPerformed(ActionEvent evt)
    {
      app.setChoice(btnNb);
      if (isPreview)
      {
        app.setState(State.DRAWING);
        Monitor.wakeUp();
      }
    }
  }

  class CheckBoxActionAdapter implements ActionListener
  {
    public void actionPerformed(ActionEvent evt)
    {
      if (checkBox.isSelected())
      {
        isPreview = true;
        app.setState(State.DRAWING);
        Monitor.wakeUp();
      }
      else
        isPreview = false;
    }
  }

  class ApplyBtnActionAdapter implements ActionListener
  {
    public void actionPerformed(ActionEvent evt)
    {
      app.setState(State.DRAWING);
      Monitor.wakeUp();
    }
  }

  private JPanel contentPane = (JPanel)getContentPane();

  private JRadioButton[] radioBtn = {new JRadioButton("Ramp"),
                                     new JRadioButton(
                                       "Rectangle"),
                                     new JRadioButton(
                                       "Triangle"),
                                     new JRadioButton(
                                       "Equal harmonics")};
  private ButtonGroup buttonGroup = new ButtonGroup();
  private JCheckBox checkBox = new JCheckBox("Preview", true);
  private JButton applyBtn = new JButton("Apply");
  private boolean isPreview = true;
  private int choice;
  private FourierEx3 app;

  SelectionPalette(FourierEx3 app)
  {
    super(app.getPanel().getWindow(), "Select"); // Modeless
    this.app = app;
    init();
    pack();
    setLocationRelativeTo(app.getPanel().getWindow());
  }

  private void init()
  {
    contentPane.setLayout(new BoxLayout(contentPane,
                                        BoxLayout.Y_AXIS));
    for (int nb = 0; nb < radioBtn.length; nb++)
    {
      buttonGroup.add(radioBtn[nb]);
      contentPane.add(radioBtn[nb]);
      radioBtn[nb].addActionListener(
        new RadioBtnActionAdapter(nb));
    }
    contentPane.add(checkBox);
    contentPane.add(applyBtn);
    checkBox.addActionListener(new CheckBoxActionAdapter());
    applyBtn.addActionListener(new ApplyBtnActionAdapter());
  }

  int getChoice()
  {
    return choice;
  }

  void show(int selection)
  {
    radioBtn[selection].setSelected(true);
    choice = selection;
    setVisible(true);
  }
}
