// DrunkenMan.java

import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

public class DrunkenMan extends Applet implements Runnable
{
  private BufferedImage bi;
  private Graphics2D offG2D;
  private int width, height;
  private int size;
  private int x0, y0, x, y, xOld, yOld;
  private int duration; // in msec
  private volatile int time; // in msec
  private int step;
  private int delay; // in msec
  private Color bkColor = new Color(0, 96, 0);
  private Color traceColor = Color.white;
  private Color manColor = Color.red;
  private boolean isStandalone = false;
  private Frame f;

  public String getParameter(String key, String def)
  {
    return isStandalone
           ? System.getProperty(key, def)
           : (getParameter(key) != null
              ? getParameter(key) : def);
  }

  public void init()
  {
    // Get parameters
    width = Integer.parseInt(getParameter("width", "300"));
    height = Integer.parseInt(getParameter("height", "300"));
    duration = Integer.parseInt(getParameter("duration",
                                             "10000"));
    step = Integer.parseInt(getParameter("step", "10"));
    size = Integer.parseInt(getParameter("size", "8"));
    delay = Integer.parseInt(getParameter("delay", "100"));

    // Create offscreen buffer
    bi = new BufferedImage(width, height,
                           BufferedImage.TYPE_INT_RGB);
    offG2D = bi.createGraphics();

    // Set background color
    offG2D.setColor(bkColor);
    offG2D.fillRect(0, 0, width, height);

    // Initialize variables
    time = 0;
    x0 = width / 2;
    y0 = height / 2;
    x = xOld = x0;
    y = yOld = y0;
  }

  public void initStandalone()
  {
    init();
    f = new Frame();
    f.setTitle("Drunken man's walk");
    f.add(this);
    f.setResizable(false);
    f.pack();
    f.setVisible(true);
    f.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent evt)
      {
        System.exit(0);
      }
    });
    start();
  }

  public void start()
  {
    new Thread(this).start();
  }

  public void stop()
  {
    time = duration; // Causes thread to stop
  }

  public void run()
  {
    while (time < duration)
    {
      try
      {
        Thread.sleep(delay);
      }
      catch (Exception ex) {}
      double angle = 2 * Math.PI * Math.random();
      x = (int)(xOld + step * Math.cos(angle));
      y = (int)(yOld + step * Math.sin(angle));
      time += delay;
      if (!isStandalone)
        showStatus("Time: " + time);
      offG2D.setColor(traceColor);
      offG2D.drawLine(xOld, yOld, x, y);
      repaint();
    }
    String msg = "Distance after " + time + " ms:  " +
                 Math.sqrt((x - x0) * (x - x0) +
                           (y - y0) * (y - y0));
    if (isStandalone)
      f.setTitle(msg);
    else
      showStatus(msg);
  }

  public void paint(Graphics g)
  {
    g.drawImage(bi, 0, 0, null);
    g.setColor(manColor);
    g.fillOval(x - size / 2, y - size / 2, size, size);
    xOld = x;
    yOld = y;
  }

  public void update(Graphics g)
  {
    paint(g);
  }

  public Dimension getPreferredSize()
  {
    return new Dimension(width, height);
  }

  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        DrunkenMan man = new DrunkenMan();
        man.isStandalone = true;
        man.initStandalone();
      }
    });
  }
}
