// PopSimApplet.java
// Not all Swing methods are invoked in
// Event Dispatch Thread (EDT) when run standalone

import javax.swing.*;

public class PopSimApplet extends JApplet implements Runnable
{
  private State s = new State(State.RESET);
  private Population p; // Could be local in PopSim()
  private boolean isStandalone = false;
  private PopDialog dlg; // same

  public void start()
  {
    new Thread(this).start();
  }

  public void stop()
  {
    s.set(State.EXIT);
  }

  public void run()
  {
    int nbPop = 40;
    int nbSick = 2; // Default
    int genTime = 10; // Default
    dlg = new PopDialog(s);
    dlg.setTextField(0, "" + nbSick);
    dlg.setTextField(1, "" + genTime);

    while (s.get() != State.EXIT)
    {
      switch (s.get())
      {
        case State.IDLE :
        case State.HALT :
          delay(1);
          break;

        case State.INIT :
          dlg.enableInput(false);
          nbSick = dlg.getInputValue(0, 1, 40);
          genTime = dlg.getInputValue(1, 1, 100);
          if (nbSick == 0 || genTime == 0 || nbSick > nbPop)
          {
            dlg.showStatus("Ungltige Anfangswerte");
            dlg.enableInput(true);
            s.set(State.IDLE);
          }
          else
          {
            p = new Population(nbPop, nbSick);
            dlg.showText(p.getPopDescription());
            dlg.showStatus(p.getPopStatus());
            if (nbSick < nbPop)
              s.set(State.GO);
            else
              s.set(State.HALT);
          }
          break;

        case State.GO :
          delay(10 * genTime);
          p.nextGen();
          if (p.getSick() == nbPop)
            s.set(State.HALT);
          dlg.showText(p.getPopDescription());
          dlg.showStatus(p.getPopStatus());
          break;

        case State.RESET :
          dlg.clear();
          dlg.enableInput(true);
          dlg.showStatus(
            "Anfangswerte eingeben (Anzahl Individuen: " +
            nbPop + ") ");
          s.set(State.IDLE);
          break;
      }
    }
    if (isStandalone)
      System.exit(0);
    else
      dlg.dispose();
  }

  private void delay(int time)
  {
    try
    {
      Thread.currentThread().sleep(time);
    }
    catch (InterruptedException ex) {}
  }

  public static void main(String[] args)
  {
    PopSimApplet psa = new PopSimApplet();
    psa.isStandalone = true;
    psa.start();
  }
}
