// ControlApplet.java
// Embedded applet with controls using GPanel

/*
HTML tag:

<applet code="ControlApplet.class"
             archive="aplu5.jar, ControlApplet$ButtonAdapter.class"
             width="500" height="500"></applet>
*/

import java.awt.Color;
import javax.swing.*;
import java.awt.*;
import ch.aplu.util.*;
import java.awt.event.*;

public class ControlApplet extends JApplet implements Runnable
{

  // ----------- Inner class ButtonAdapter ---------------------------
  private class ButtonAdapter implements ActionListener
  {
    public void actionPerformed(ActionEvent evt)
    {
      // Text entry should be validated, see IntegerValidator in kap29
      speed = Integer.valueOf(textField.getText().trim()).intValue();
      Object source = evt.getSource();
      if (source == startButton)
      {
        isRunning = true;
        size = startSize;
        step = startStep;
      }
      if (source == stopButton)
        isRunning = false;
      if (source == resumeButton)
        isRunning = true;
     }
  }

  // ------------- Constants -----------------------------------------
  private final int max = 500;
  private final int min = 50;
  private final int defaultSpeed = 10;
  private final int startSize = 100;
  private final int startStep = 10;
  private final Color bgColor = Color.white;

  // ------------- Instance variables --------------------------------
  private GPane p;
  private JTextField textField = new JTextField(3);
  private JLabel label = new JLabel("Speed (1..100):");
  private JButton startButton = new JButton("Start");
  private JButton stopButton = new JButton("Stop");
  private JButton resumeButton = new JButton("Resume");
  private boolean isStandalone = false;

  // Volatile because two threads use them
  private volatile boolean isRunning;
  private volatile boolean isActive;
  private volatile int size;
  private volatile int speed;
  private volatile int step;

  // ------------- Override methods of JApplet -----------------------
  public void start()
  {
    if (isStandalone)
      p = new GPane(new Size(500, 500));
    else
      p = new GPane();
    p.bgColor(bgColor);
    p.window(0, max, 0, max);
    p.add(label);
    p.add(textField);
    p.add(startButton);
    p.add(stopButton);
    p.add(resumeButton);
    startButton.addActionListener(new ButtonAdapter());
    stopButton.addActionListener(new ButtonAdapter());
    resumeButton.addActionListener(new ButtonAdapter());
    textField.setText(Integer.toString(defaultSpeed));
    speed = defaultSpeed;
    isRunning = false;
    isActive = true;
    size = startSize;
    step = startStep;
    getContentPane().add(p);
    new Thread(this).start();
  }

  public void stop()
  {
    isActive = false;
    p.dispose();
  }

  // ------------- Implement run() of Runnable -----------------------
  public void run()
  {
    clearGraphics();
    p.enableRepaint(false);
    while (isActive)
    {
      Thread.yield();
      if (isRunning)
      {
        clearGraphics();
        p.rectangle(size, size);
        p.repaint();
        if (size == max || size == min)
          step = -step;
        size += step;
        Console.delay(1000/speed);
      }
    }
  }

  // ------------- private method(s) ---------------------------------
  private void clearGraphics()
  {
    p.clear();  // erases graphics buffer only, because of enableRepaint(false)
//    p.move(0,0); // not needed because clear() does it
    p.draw(max, max);
    p.move(0, max);
    p.draw(max, 0);
    p.move(max/2, max/2);
    p.rectangle(max, max);
  }

  // ------------- main() for standalone testing ---------------------
  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        ControlApplet ca = new ControlApplet();
        ca.isStandalone = true;
        ca.start();
        JFrame f = new JFrame();
        f.setTitle("ControlApplet");
        f.setContentPane(ca.getContentPane());
        f.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        f.pack();
        f.setResizable(false);
        f.setVisible(true);
      }
    });
  }
}
