// DbEx2.java
// Compile J2SE V5 up

import java.awt.EventQueue;
import java.sql.*;

enum State
{
  NAVIGATE,
  INSERT,
  SEARCH;
}

public class DbEx2
{
  private State state = State.NAVIGATE;
  private final String host = "localhost";
  private final String database = "demo";
  private final String table = "buch";
  private final String username = "root"; // Use your MySQL account name
  private final String password = "samsung"; // Use the corresponding password

  private DbDialog dlg;
  private Connection con;
  private Statement stmt;
  private ResultSet rs;
  private int nbRecords;
  private String searchString = "";

  public DbEx2()
  {
    try
    {
      Class.forName("org.gjt.mm.mysql.Driver");
    }
    catch (ClassNotFoundException ex)
    {
      System.out.println("Unable to load driver class");
      return;
    }

    try
    {
      String drv = "jdbc:mysql://" + host + "/" + database;
      con = DriverManager.getConnection(drv, username, password);
      stmt =
        con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,
                            ResultSet.CONCUR_UPDATABLE);
      getRecords();
      dlg = new DbDialog("Bcherdatenbank", this);
      if (nbRecords > 0)
      {
        dlg.showRecord(rs);
        dlg.showStatus(nbRecords + " record(s) selected");
      }
      else
        insertRecord(true);
    }
    catch (SQLException ex)
    {
      System.out.println("Sorry. Got SQLException: " +
                         ex.getMessage());
      releaseDb();
    }
  }

  void doIt(int n)
  {
    try
    {
      switch (n)
      {
        case 0 : // First
          updateRecord();
          rs.first();
          dlg.showRecord(rs);
          break;

        case 1 : // Prev
          updateRecord();
          if (!rs.isFirst())
            rs.previous();
          dlg.showRecord(rs);
          break;

        case 2 : // Next
          updateRecord();
          if (!rs.isLast())
            rs.next();
          dlg.showRecord(rs);
          break;

        case 3 : // Last
          updateRecord();
          rs.last();
          dlg.showRecord(rs);
          break;

        case 4 : // Delete
          deleteRecord();
          break;

        case 5 : // Insert
          updateRecord();
          insertRecord(true);
          break;

        case 6 : // Save
          if (state == State.INSERT)
            saveInsert();
          else
            updateRecord();
          break;

        case 7 : // Search
          state = State.SEARCH;
          searchRecord();
          break;

        case 8 : // Exit
          releaseDb();
          System.exit(0);
          break;
      }
    }
    catch (SQLException ex)
    {
      System.out.println("Sorry. Got SQLException: " +
                         ex.getMessage());
      releaseDb();
    }
  }

  private void deleteRecord() throws SQLException
  {
    int row = rs.getRow();
    int id = rs.getInt(1);
    if (rs.isLast())
      row--;

    String sql = "DELETE FROM " + table + " WHERE id = " + id;
    stmt.executeUpdate(sql);
    state = State.NAVIGATE;
    dlg.setSearchString("");
    getRecords();
    dlg.showRecord(rs);
    if (nbRecords > 0)
    {
      dlg.showStatus("Record deleted - " + nbRecords +
                     " record(s) selected");
    }
    else
      insertRecord(true);
  }

  private void insertRecord(boolean b)
  {
    if (b)
    {
      state = State.INSERT;
      dlg.clearFields();
      dlg.showStatus("Insert mode");
    }
    else
    {
      state = State.NAVIGATE;
      dlg.showStatus("");
    }
    dlg.setButtonsDisabled();
  }

  private void saveInsert() throws SQLException
  {
    String[] fields = dlg.getFields();
    String sql = "";
    boolean isError = false;
    try
    {
      sql = "INSERT INTO " + table +
            "(autor, titel, jahr, verlag, preis) VALUES ('" +
            fields[0] + "', '" + fields[1] + "', " +
            Integer.parseInt(fields[2]) + ", '" + fields[3] +
            "', " + Double.parseDouble(fields[4]) + ")";
      stmt.executeUpdate(sql);
    }
    catch (NumberFormatException ex)
    {
      isError = true;
    }
    insertRecord(false);
    getRecords();
    dlg.showRecord(rs);
    if (isError)
    {
      if (nbRecords == 0)
        insertRecord(true);
      dlg.showStatus(
        "Illegal format - record not inserted. " +
        nbRecords + " record(s) selected");
    }
    else
      dlg.showStatus("Record inserted - " + nbRecords +
                     " record(s) selected");
  }

  private void updateRecord() throws SQLException
  {
    String[] fields = dlg.getFields();
    int row = rs.getRow();
    int id = rs.getInt(1);
    String sql = "";
    try
    {
      sql = "UPDATE " + table + " SET autor = '" +
            fields[0] + "', titel = '" + fields[1] +
            "', jahr = " + Integer.parseInt(fields[2]) +
            ", verlag = '" + fields[3] + "', preis = " +
            Double.parseDouble(fields[4]) + " WHERE id=" + id;
    }
    catch (NumberFormatException ex)
    {
      dlg.showStatus("Illegal format - record not updated");
      return;
    }
    stmt.executeUpdate(sql);
    getRecords();
    dlg.showStatus(nbRecords + " record(s) selected");
    rs.absolute(row);
  }

  private void searchRecord() throws SQLException
  {
    searchString = dlg.getSearchString();
    int total = nbRecords;
    getRecords();
    if (nbRecords == 0)
    {
      if (searchString.equals("*"))
        dlg.showStatus("All records selected");
      else
        dlg.showStatus(
          "No records found - all records selected");
      dlg.setSearchString("");
      state = State.NAVIGATE;
      getRecords();
      dlg.showRecord(rs);
    }
    else
    {
      dlg.showRecord(rs);
      dlg.showStatus(
        nbRecords + " of " + total +
        " record(s) selected. Search * to get all records");
    }
  }

  private void getRecords() throws SQLException

  // Set instance variables rs and nbRecords
  {
    String sql;
    if (state == State.SEARCH)
      sql = "SELECT * FROM " + table + " WHERE autor='" +
            searchString + "' ORDER BY autor";
    else
      sql = "SELECT * FROM " + table + " ORDER BY autor";
    rs = stmt.executeQuery(sql); // Does not modify current rs
    if (rs.last())
      nbRecords = rs.getRow();
    else
      nbRecords = 0;
    rs.first();
  }

  State getState()
  {
    return state;
  }

  int getNbRecords()
  {
    return nbRecords;
  }

  void releaseDb()
  {
    try
    {
      if (con != null)
        con.close();
    }
    catch (SQLException ex) {}
  }

  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        new DbEx2();
      }
    });
  }
}
